/**
 *
 *  You can modify and use this source freely
 *  only for the development of application related Live2D.
 *
 *  (c) Live2D Inc. All rights reserved.
 */

#import "ViewController.h"
#import "Live2DModelIPhone.h"
#import "util/UtSystem.h"

using namespace live2d;

@interface ViewController ()

@property (strong, nonatomic) EAGLContext *context;

- (void)setupGL;
- (void)tearDownGL;

@end

@implementation ViewController
{
	Live2DModelIPhone* live2DModel ;
	NSMutableArray* textures ;
}

- (void)dealloc
{
	// Live2Dモデルの解放
	delete live2DModel;
	
	// テクスチャを解放
	for (int i=0; i<[textures count]; i++)
	{
		GLuint glTexNo=[[textures objectAtIndex:i] intValue];
		glDeleteTextures(1,&glTexNo);
	}

    [self tearDownGL];
    
    if ([EAGLContext currentContext] == self.context) {
        [EAGLContext setCurrentContext:nil];
    }
    
    [_context release];
    [super dealloc];
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    self.context = [[[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES1] autorelease];

    if (!self.context) {
        NSLog(@"Failed to create ES context");
    }
    
    GLKView *view = (GLKView *)self.view;
    view.context = self.context;
    view.drawableDepthFormat = GLKViewDrawableDepthFormat24;
    
    [self setupGL];
	
	NSString* MODEL_FILE = @"haru" ;
	NSString* TEXTURE_PATH[] ={
		@"texture_00" ,
		@"texture_01" ,
		@"texture_02" ,
		NULL
	} ;
	
	NSString *modelpath = [[NSBundle mainBundle] pathForResource:MODEL_FILE ofType:@"moc"];
	
	live2DModel = Live2DModelIPhone::loadModel( [modelpath UTF8String] ) ;
	
	for( int i = 0 ; TEXTURE_PATH[i] != NULL ; i++ ){
		NSString* texturePath = [[NSBundle mainBundle] pathForResource:TEXTURE_PATH[i] ofType:@"png"];
		GLKTextureInfo* textureInfo =
		[GLKTextureLoader textureWithContentsOfFile:texturePath
											options:[NSDictionary dictionaryWithObjectsAndKeys:
													 [NSNumber numberWithBool:YES] ,GLKTextureLoaderApplyPremultiplication,
													 [NSNumber numberWithBool:YES] ,GLKTextureLoaderGenerateMipmaps,nil
													 ]
											  error:nil];
				
		int glTexNo = [textureInfo name] ;
		live2DModel->setTexture( i , glTexNo ) ;// テクスチャとモデルを結びつける
		[textures addObject:[NSNumber numberWithInt:glTexNo]];// 解放するために設定
	}
	
	float modelWidth = live2DModel->getCanvasWidth(); // モデルの幅
    float width = [[UIScreen mainScreen] bounds].size.width;
	float height = [[UIScreen mainScreen] bounds].size.height;
	//  描画範囲の設定
	glOrthof(
             0,
             modelWidth,
             modelWidth / (width/height),
             0,
             0.5f, -0.5f
             );
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];

    if ([self isViewLoaded] && ([[self view] window] == nil)) {
        self.view = nil;
        
        [self tearDownGL];
        
        if ([EAGLContext currentContext] == self.context) {
            [EAGLContext setCurrentContext:nil];
        }
        self.context = nil;
    }

    // Dispose of any resources that can be recreated.
}

- (void)setupGL
{
    [EAGLContext setCurrentContext:self.context];
}

- (void)tearDownGL
{
    [EAGLContext setCurrentContext:self.context];
}

#pragma mark - GLKView and GLKViewController delegate methods

- (void)glkView:(GLKView *)view drawInRect:(CGRect)rect
{
    glClearColor(0.65f, 0.65f, 0.65f, 1.0f);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	double t = UtSystem::getUserTimeMSec()/1000.0 ;
	live2DModel->setParamFloat("PARAM_ANGLE_X", 30 * sin( t ) );
	
	live2DModel->update() ;
	live2DModel->draw() ;
    
}

@end
